Con
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  CLK_FREQ = ((_clkmode-xtal1)>>6)*_xinfreq
  MS_001 = CLK_FREQ / 1_000  

  ' // Propeller Pinout

  Misc1 = 0
  Misc2 = 1
  Misc3 = 2
  Misc4 = 3

  MCPCLK  = 4
  MCPDOut = 5
  MCPDIn  = 5
  MCPCS   = 6      

  NC1 = 7
 
  MPUsda  = 8       ' All I2C Sensors on SAME Bus
  MPUscl  = 9
  
  RPitx  = 10        
  RPirx  = 11
  
  GPSrx   = 12  ' SCL
  GPStx   = 13  ' SDA

  BMPSCL   = 12   
  BMPSDA   = 13
                  
  WiFitx   = 14       ' White
  WiFirx   = 15       ' Green     

  Misc5 = 16    
  Misc6 = 17
  Misc7 = 18
              
  LED   = 19
  Piezo = 20

  Servo6 = 21        ' Left Side Airlerons
  Servo5 = 22        ' Gyro Select
  Servo4 = 23        ' Gyro / Rudder

  NC2 = 24
                     
  Servo3 = 25        ' Motor Speed / Power
  Servo2 = 26        ' Elevator
  Servo1 = 27        ' Right Side Airlerons

  EEPROMscl   = 28
  EEPROMsda   = 29

  DEBUGSERtx  = 30
  DEBUGSERrx  = 31


  SCL = 12

  ' // BMP-180 Settings
  bmpAddr  = %1110_1110
  
  bmpAddrR = %1110_1110
  bmpAddrW = %1110_1111
  
  mpuAddr  = %1101_0000
  
  OSS1 = 0      ' Ultra Low Power
  OSS2 = 1      ' Standard
  OSS3 = 2      ' High Resolution
  OSS4 = 3      ' Ultra High Resolution


  ' EEPROM data base address - i.e 32K (assumes a 64kb eeprom installed!!)
  EEPROM_Base   = $8000  
  EEPROMAddr    = %1010_0000  

  i2cSCL        = 28
'  i2cSDA        = 29   

OBJ

  i2c           :       "pasm_i2c_driver_Ext"         ' Finds Both Devices, All Data = FF
  'i2c           :       "basic_i2c_driver_1.3_Mod"   ' Finds MPU Only, Data = 255
  Ser           :       "FullDuplexSerial" 


Var
  long ac1, ac2, ac3, ac4, ac5, ac6, b1, b2, mb, mc, md, id
  long UT, UP
  

  long T, P
  long i2cAddress, i2cSlaveCounter

  long oss

Pub Start
  ser.start(31,30,0,115200)

  i2c.Initialize(mpuSCL)                 ' // PASM i2c Driver
  'i2c.Initialize(mpuSCL, mpuSDA)        ' // basic_i2c_driver1.3_Mod

  oss := OSS2
  
  repeat

    ' // All I2C Sensors on SAME Bus

    ' // MPU-6050
    if i2c.devicePresent(mpuSCL, mpuAddr) == True
      ser.str(string("MPU - Device Located!"))
      ser.tx(13)
      waitcnt(clkfreq + cnt)
      GetMPU
    else
      ser.str(string("MPU - Device Not Found"))
      ser.tx(13)
      waitcnt(clkfreq + cnt)


    ' // BMP-180
    if i2c.devicePresent(mpuSCL, bmpAddr) == True
      ser.str(string("BMP - Device Located!"))
      ser.tx(13)
      waitcnt(clkfreq + cnt)
      GetBMP
    else
      ser.str(string("BMP - Device Not Found"))
      ser.tx(13)
      waitcnt(clkfreq + cnt)

    

Pub GetMPU | X, Y, Z, A, B, C
  'repeat
    ' // Setup: power mgmt, dlp, smplrtdiv, gyro & accel
    i2c.writeLocation(mpuSCL, mpuAddr, 107, %00000001)    ' X Gyro as Clock Source
    i2c.writeLocation(mpuSCL, mpuAddr, 26, %00000011)     ' 40 Hz DLP
    i2c.writeLocation(mpuSCL, mpuAddr, 25, %00000001)     ' Sample Rate = 500 Hz
    i2c.writeLocation(mpuSCL, mpuAddr, 27, %00011000)     ' Gyro  = FS3
    i2c.writeLocation(mpuSCL, mpuAddr, 28, %00010000)     ' Accel = AFS2
     
    ' // Read Values
    X := i2c.readLocation(mpuSCL, mpuAddr, 59)   ' = 255
     
    ser.str(string("MPU Out: "))
    ser.dec(X)
    ser.tx(13)
     
      
Pub GetBMP | v1, v2, v3, up1, up2, up3, ut1, ut2
  'repeat
    
      {
      Table 5: Calibration Coefficients
                  BMP180 reg adr
      Parameter   MSB        LSB
        AC1       0xAA       0xAB
        AC2       0xAC       0xAD
        AC3       0xAE       0xAF
        AC4       0xB0       0xB1
        AC5       0xB2       0xB3
        AC6       0xB4       0xB5
        B1        0xB6       0xB7
        B2        0xB8       0xB9
        MB        0xBA       0xBB
        MC        0xBC       0xBD
        MD        0xBE       0xBF
      }

    ' i2c.readLocation(SCL,device_address, register)

    ' // Read Calibration Coefficients
    ac1 := i2c.readLocation(mpuSCL, bmpAddr, $AA)  ' 1B
    ac2 := i2c.readLocation(mpuSCL, bmpAddr, $AC)  ' FB
    ac3 := i2c.readLocation(mpuSCL, bmpAddr, $AE)  ' C7
    ac4 := i2c.readLocation(mpuSCL, bmpAddr, $B0)  ' 84
    ac5 := i2c.readLocation(mpuSCL, bmpAddr, $B2)  ' 61
    ac6 := i2c.readLocation(mpuSCL, bmpAddr, $B4)  ' 49

    b1 := i2c.readLocation(mpuSCL, bmpAddr, $B6)   ' 19
    b2 := i2c.readLocation(mpuSCL, bmpAddr, $B8)   ' 00

    mb := i2c.readLocation(mpuSCL, bmpAddr, $BA)   ' 80
    mc := i2c.readLocation(mpuSCL, bmpAddr, $BC)   ' D1
    md := i2c.readLocation(mpuSCL, bmpAddr, $BE)   ' 0A

    id := i2c.readLocation(mpuSCL, bmpAddr, $D0)   ' 55     *** Should = 0x55
     
    waitcnt((clkfreq / 4) + cnt)

    ser.str(string("AC1: "))
    ser.hex(ac1, 2)
    ser.str(string("  "))
    ser.dec(ac1)
    ser.tx(13)
    ser.str(string("AC2: "))
    ser.hex(ac2, 2)
    ser.str(string("  "))
    ser.dec(ac2)    
    ser.tx(13)
    ser.str(string("AC3: "))
    ser.hex(ac3, 2)
    ser.str(string("  "))
    ser.dec(ac3)    
    ser.tx(13)
    ser.str(string("AC4: "))
    ser.hex(ac4, 2)
    ser.str(string("  "))
    ser.dec(ac4)    
    ser.tx(13)
    ser.str(string("AC5: "))
    ser.hex(ac5, 2)
    ser.str(string("  "))
    ser.dec(ac5)    
    ser.tx(13)
    ser.str(string("AC6: "))
    ser.hex(ac6, 2)
    ser.str(string("  "))
    ser.dec(ac6)    
    ser.tx(13)                    

    ser.str(string("B1: "))
    ser.hex(b1, 2)
    ser.str(string("  "))
    ser.dec(b1)    
    ser.tx(13)
    ser.str(string("B2: "))
    ser.hex(b2, 2)
    ser.str(string("  "))
    ser.dec(b2)    
    ser.tx(13)

    ser.str(string("MB: "))
    ser.hex(mb, 2)
    ser.str(string("  "))
    ser.dec(mb)    
    ser.tx(13)
    ser.str(string("MC: "))
    ser.hex(mc, 2)
    ser.str(string("  "))
    ser.dec(mc)    
    ser.tx(13)
    ser.str(string("MD: "))
    ser.hex(md, 2)
    ser.str(string("  "))
    ser.dec(md)
    ser.tx(13)        

    ser.str(string("ID: "))
    ser.hex(id, 2)
    ser.str(string("  "))
    ser.dec(id)    
    ser.tx(13)
    
    'ser.str(string("bmpAddr: "))
    'ser.hex(bmpAddr | 1, 2)        ' 0 = EE, 1 = EF
    'ser.tx(13)
    
    ' // Read Uncompressed Temperature Value
    ' Write 0x2E into reg 0xF4, wait 4.5ms
    ' Read reg 0xF6 (MSB), 0xF7 (LSB)

    i2c.writeLocation(mpuSCL, bmpAddrW, $F4, $2E)       ' Temperature  0x2E  4.5
    waitcnt((clkfreq / 222) + cnt)             ' 4.5ms
    ut1 := i2c.readLocation(mpuSCL, bmpAddr, $F6)
    ut2 := i2c.readLocation(mpuSCL, bmpAddr, $F7) 

    'UT := i2c.readLocation16(mpuSCL, bmpAddr, i2c#OneAddr|$F6)

    ser.str(string("UT: "))
    ser.hex(ut1, 2)
    ser.str(string("  "))
    ser.hex(ut2, 2)
    ser.str(string("  "))    
    ser.dec(ut1 + ut2)
    ser.tx(13)
    
    ' // Read Uncompressed Pressure Value
    ' Write 0x34+(oss<<6) into reg 0xF4, wait #.#ms
    ' Read reg 0xF6 (MSB), 0xF7 (LSB), 0xF8 (XLSB)

    'i2c.writeLocation(mpuSCL, bmpAddr, $F4, $34 + (oss<<6))

    ' i2c.readLocation(SCL,device_address, register)
    ' i2c.writeLocation(SCL,device_address, register, value) 
    
    if oss == 0
      i2c.writeLocation(mpuSCL, bmpAddrW, $F4, $34)     ' Pressure (oss = 0)  0x34  4.5
      waitcnt((clkfreq / 222) + cnt)                         ' 4.5ms    
    elseif oss == 1
      i2c.writeLocation(mpuSCL, bmpAddrW, $F4, $74)     ' Pressure (oss = 1)  0x74  7.5
      waitcnt((clkfreq / 133) + cnt)                         ' 7.5ms    
    elseif oss == 2
      i2c.writeLocation(mpuSCL, bmpAddrW, $F4, $B4)     ' Pressure (oss = 2)  0xB4  13.5
      waitcnt((clkfreq / 74) + cnt)                          ' 13.5ms   
    elseif oss == 3
      i2c.writeLocation(mpuSCL, bmpAddrW, $F4, $F4)     ' Pressure (oss = 3)  0xF4  25.5
      waitcnt((clkfreq / 39) + cnt)                          ' 25.5ms

    up1 := i2c.readLocation(mpuSCL, bmpAddr, $F6)
    up2 := i2c.readLocation(mpuSCL, bmpAddr, $F7)
    up3 := i2c.readLocation(mpuSCL, bmpAddr, $F8)         
    
    ser.str(string("UP: "))
    ser.hex(up1 + up2 + up3, 6)
    ser.str(string("  "))
    ser.dec(up1 + up2 + up3)    
    ser.tx(13)
    ser.str(string("UP 123 Hex: "))  
    ser.hex(up1, 2)
    ser.str(string("  "))
    ser.hex(up2, 2)
    ser.str(string("  "))
    ser.hex(up3, 2)
    ser.tx(13)        
    ser.str(string("UP 123 Dec: "))  
    ser.dec(up1)
    ser.str(string("  "))
    ser.dec(up2)
    ser.str(string("  "))
    ser.dec(up3)
    ser.tx(13)        


    ' // Calculate True Temperature

    ' // Calculate True Pressure


    waitcnt(clkfreq + cnt)

    